# A test for encryption (#27: https://forge.ipsl.jussieu.fr/epoll/ticket/27)

use Crypt::RSA; # crypt/decrypt
use Crypt::CBC; # sym key
use XML::Simple; # dump data
use MIME::Base64; # to make it ascii

my $rsa = new Crypt::RSA ES => 'PKCS1v15'; 

    my ($public, $private) = 
        $rsa->keygen ( 
            Identity  => 'Vote ID 12',
            Size      => 1024,  
            Password  => undef, 
            Verbosity => 0,
            KF=>'SSH',
        ) or die $rsa->errstr();
print $public->serialize;
print "\n";
print encode_base64($private->serialize);
print "\n";

my $symkey = map{ chr(rand(256)) } (1 .. (256 / 8));

my $xs = XML::Simple->new(ForceArray => 1, RootName => 'ballot');
print my $xml = $xs->XMLout({ id => '8daad55a7f94a4fb79bda7ab007953dc', sbal => [ 'coco', 'kikikikikikikikikikikikiki' ], fsbal => [] });
print "# ENC SYMKEY\n";

my $encryptedsymkey = $rsa->encrypt ( 
            Message    => $symkey,
            Key        => $public,
            Armour     => 1,
        ) || die $rsa->errstr();

# reading
print "# DEC SYMKEY\n";
$symkey = $rsa->decrypt(
        Cyphertext => $encryptedsymkey,
            Key        => $private,
            Armour     => 1,
        ) || die $rsa->errstr();

my $ballot;
my $cipher = new Crypt::CBC($symkey, 'DES');
print "# ENC BALLOT\n";
my $encryptedballot = $cipher->encrypt_hex($xml);
print $encryptedballot . "\n";

print "# DEC BALLOT\n";
foreach (0..1000) {
    print "$_\n";
    $symkey = $rsa->decrypt(
        Cyphertext => $encryptedsymkey,
            Key        => $private,
            Armour     => 1,
        ) || die $rsa->errstr();
    my $cipher = new Crypt::CBC($symkey, 'DES');
    $ballot = $cipher->decrypt_hex($encryptedballot);
}

print $ballot

